@@ -5,9 +5,10 @@
  * to create the user's ticket cache.  The shared code is abstracted here into
  * the pamk5_setcred function.
  *
+ * Copyright 2005, 2006, 2007, 2008, 2009, 2014, 2017
+ *     Russ Allbery <eagle@eyrie.org>
  * Copyright 2011
  *     The Board of Trustees of the Leland Stanford Junior University
- * Copyright 2005, 2006, 2007, 2008, 2009, 2014 Russ Allbery <eagle@eyrie.org>
  * Copyright 2005 Andres Salomon <dilinger@debian.org>
  * Copyright 1999, 2000 Frank Cusack <fcusack@fcusack.com>
  *
@@ -264,7 +265,7 @@ pamk5_setcred(struct pam_args *args, bool refresh)
      * with its weird PAM handling, so we're going to cobble up a new context
      * for ourselves.
      */
-    pamret = pamk5_context_fetch(args);
+    pamk5_context_fetch(args);
     if (args->config->ctx == NULL) {
         putil_debug(args, "no context found, creating one");
         pamret = create_session_context(args);
