@@ -75,13 +75,26 @@ main(void)
 
     plan_lazy();
 
-    /* Default behavior. */
+    /*
+     * Default behavior.  We have to distinguish between two versions of
+     * Heimdal for testing because the prompts changed substantially.  Use the
+     * existence of krb5_principal_set_comp_string to distinguish because it
+     * was introduced at the same time.
+     */
 #ifdef HAVE_KRB5_HEIMDAL
+# ifdef HAVE_KRB5_PRINCIPAL_SET_COMP_STRING
     run_script("data/scripts/expired/basic-heimdal", &config);
     config.newpass = krbconf->password;
     config.password = newpass;
     kerberos_expire_password(krbconf->userprinc, now);
     run_script("data/scripts/expired/basic-heimdal-debug", &config);
+# else
+    run_script("data/scripts/expired/basic-heimdal-old", &config);
+    config.newpass = krbconf->password;
+    config.password = newpass;
+    kerberos_expire_password(krbconf->userprinc, now);
+    run_script("data/scripts/expired/basic-heimdal-old-debug", &config);
+# endif
 #else
     run_script("data/scripts/expired/basic-mit", &config);
     config.newpass = krbconf->password;
@@ -122,17 +135,27 @@ main(void)
     run_script("data/scripts/expired/fail", &config);
     run_script("data/scripts/expired/fail-debug", &config);
 
-    /* Defer the error to the account management check. */
+    /*
+     * Defer the error to the account management check.
+     *
+     * Skip this check on Heimdal currently (Heimdal 7.4.0) because its
+     * implementation of krb5_get_init_creds_opt_set_change_password_prompt is
+     * incomplete.  See <https://github.com/heimdal/heimdal/issues/322>.
+     */
+# ifdef HAVE_KRB5_HEIMDAL
+    skip_block(2, "deferring password changes broken in Heimdal");
+# else
     config.newpass = newpass;
     config.password = krbconf->password;
     config.authtok = krbconf->password;
     kerberos_expire_password(krbconf->userprinc, now);
-    run_script("data/scripts/expired/defer", &config);
+    run_script("data/scripts/expired/defer-mit", &config);
     config.newpass = krbconf->password;
     config.password = newpass;
     config.authtok = newpass;
     kerberos_expire_password(krbconf->userprinc, now);
-    run_script("data/scripts/expired/defer-debug", &config);
+    run_script("data/scripts/expired/defer-mit-debug", &config);
+# endif
 
 #else /* !HAVE_KRB5_GET_INIT_CREDS_OPT_SET_CHANGE_PASSWORD_PROMPT */
 
