@@ -6,9 +6,10 @@
  * only implemented functionality is to mark a password as expired.
  *
  * The canonical version of this file is maintained in the rra-c-util package,
- * which can be found at <http://www.eyrie.org/~eagle/software/rra-c-util/>.
+ * which can be found at <https://www.eyrie.org/~eagle/software/rra-c-util/>.
  *
  * Written by Russ Allbery <eagle@eyrie.org>
+ * Copyright 2017 Russ Allbery <eagle@eyrie.org>
  * Copyright 2011
  *     The Board of Trustees of the Leland Stanford Junior University
  *
@@ -68,7 +69,7 @@ kerberos_expire_password(const char *principal, time_t expires)
     krb5_context ctx;
     krb5_principal admin = NULL;
     krb5_principal princ = NULL;
-    krb5_error_code code;
+    kadm5_ret_t code;
     kadm5_config_params params;
     kadm5_principal_ent_rec ent;
     void *handle;
@@ -110,7 +111,7 @@ kerberos_expire_password(const char *principal, time_t expires)
     }
     memset(&ent, 0, sizeof(ent));
     ent.principal = princ;
-    ent.pw_expiration = expires;
+    ent.pw_expiration = (krb5_timestamp) expires;
     code = kadm5_modify_principal(handle, &ent, KADM5_PW_EXPIRATION);
     if (code == 0)
         okay = true;
